/*
 * Created on 04-Sep-2005
 * Created by Paul Gardner
 * Copyright (C) 2005 Aelitis, All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * AELITIS, SARL au capital de 30,000 euros
 * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
 *
 */

package org.parg.azureus.plugins.sudoku.model.impl;

import org.parg.azureus.plugins.sudoku.model.*;

public class 
SudokuStrategyDice
	extends SudokuStrategy
{
	private SudokuModel		model;
	
	protected
	SudokuStrategyDice(
		SudokuModelImpl		_model )
	{
		super( _model );
		
		model	= _model;
	}
	
	public boolean
	apply(
		SudokuSolveInstance	solve_instance)
	{
		boolean	did_something = false;
		
		int	model_size = model.getSize();
		
		SudokuValue[][]	values = model.getValues();
		
		for (int x=0;x<model.getWidth();x++){
			
			int	x_chunk = (x/model_size)*model_size;
			
			for (int y=0;y<model.getHeight();y++){

				int	y_chunk = (y/model_size)*model_size;
				
				SudokuValue	value = values[x][y];
				
				if ( !value.hasValue()){
					
					continue;
				}
								
				for (int xc=x_chunk;xc<x_chunk+model_size;xc++){
					
					for (int yc=y_chunk;yc<y_chunk+model_size;yc++){

						SudokuValueImpl	c_value = (SudokuValueImpl)values[xc][yc];

						if ( c_value != value ){
							
							did_something	= true;
							
							c_value.setPossible(
								solve_instance,
								value.getValue(), false,
								new SudokuPossibilityMetricImpl(
										SudokuPossibilityMetric.MT_ALREADY_IN_CHUNK,
										"already in chunk" ));
						}
					}
				}	
			}
		}
		
		return( did_something );
	}
	
	public boolean
	update(
		SudokuSolveInstance	solve_instance )
	{
		boolean	did_something	= false;
		
		int	model_size = model.getSize();
		
		SudokuValue[][]	values = model.getValues();
		
		for (int x=0;x<model.getWidth();x+=model_size){
						
			for (int y=0;y<model.getHeight();y+=model_size){
						
					// look for any entries that can either only be 1 value or are the only
					// entries that can be a given value
				
				int[]	possibility_totals = new int[model_size*model_size];
				
				SudokuValue[]	possibility_totals_vals = new SudokuValue[possibility_totals.length];
				
				int[]	weak_possibility_totals = new int[model_size*model_size];
				
				SudokuValue[]	weak_possibility_totals_vals = new SudokuValue[possibility_totals.length];

				for (int xc=x;xc<x+model_size;xc++){
					
					for (int yc=y;yc<y+model_size;yc++){

						SudokuValue	c_value = values[xc][yc];
							
						int	total = 0;
						int	last_possibility = 0;
						
						boolean[] possibilities 		= c_value.getPossibles( true );
						boolean[] weak_possibilities 	= c_value.getPossibles( false );
						
						for (int z=0;z<possibilities.length;z++){
							
							if ( possibilities[z] ){
								
								possibility_totals[z]++;
								
								possibility_totals_vals[z]	= c_value;
								
								total++;
								
								last_possibility = z;
							}
							
							if ( weak_possibilities[z] ){
								
								weak_possibility_totals[z]++;
								
								weak_possibility_totals_vals[z]	= c_value;
							}
						}
						
						if ( total == 1 ){
							
							// System.out.println( "Dicer: " + c_value.getString(true) + " can only be " + ( last_possibility+1 ) + " as it is the only choice");
							
							did_something	= true;
							
							updateValue( 
									solve_instance,
									c_value, last_possibility+1, 
									new SudokuValueMetricImpl(
											SudokuValueMetric.MT_CHUNK_ONLY_CHOICE,
											"chunk: only choice" ));
						}
					}
				}	
				
				for (int i=0;i<possibility_totals.length;i++){
					
					if ( possibility_totals[i] == 1 ){
						
						// System.out.println( "Dicer: " + possibility_totals_vals[i].getString(true) + " can only be " + ( i+1 ) + " as no others can be");
						
						did_something	= true;
						
						updateValue( 
								solve_instance,
								possibility_totals_vals[i], i+1, 
								new SudokuValueMetricImpl(
										SudokuValueMetric.MT_CHUNK_NO_OTHERS,
										"chunk: no others can be" ));
					}
				}
				
				for (int i=0;i<weak_possibility_totals.length;i++){
					
					if ( weak_possibility_totals[i] == 1 ){
						
						SudokuValue v = weak_possibility_totals_vals[i];
						
						if ( v.hasValue() && v.isExplicit()){
							
							if ( v.getValue() != ( i+1 )){
								
								// System.out.println( "Dicer: " + weak_possibility_totals_vals[i].getString(true) + " inconsistent: can only be " + ( i+1 ));
								
								did_something	= true;
								
								updateValue( 
										solve_instance,
										v, i+1, 
										new SudokuValueMetricImpl(
												SudokuValueMetric.MT_CHUNK_ONLY_CHOICE,
												"chunk: recovery" ));

							}
						}
					}
				}
			}
		}
		
		return( did_something );
	}
	
	public int
	getComplexity()
	{
		return( COMP_SIMPLE );
	}
}
