/*
 * Created on 04-Sep-2005
 * Created by Paul Gardner
 * Copyright (C) 2005 Aelitis, All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * AELITIS, SARL au capital de 30,000 euros
 * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
 *
 */

package org.parg.azureus.plugins.sudoku.model.impl;

import org.parg.azureus.plugins.sudoku.model.*;

public class 
SudokuStrategySlice
	extends SudokuStrategy
{
	private SudokuModel		model;
	
	protected
	SudokuStrategySlice(
		SudokuModelImpl		_model )
	{
		super( _model );
		
		model	= _model;
	}
	
	public boolean
	apply(
		SudokuSolveInstance	solve_instance )
	{
		boolean did_something = apply( solve_instance, model.getValues(), false );
		
		if ( apply( solve_instance, model.getReflectedValues(), true )){
			
			did_something = true;
		}
		
		return( did_something );
	}
	
	protected boolean
	apply(
		SudokuSolveInstance 	solve_instance,
		SudokuValue[][]			values,
		boolean					row )
	{	
		boolean	did_something = false;
		
		for (int x=0;x<model.getWidth();x++){
						
			for (int y=0;y<model.getHeight();y++){
				
				SudokuValue	value = values[x][y];
				
				if ( !value.hasValue()){
					
					continue;
				}
								
				for (int y2=0;y2<model.getHeight();y2++){

					SudokuValueImpl	o_value = (SudokuValueImpl)values[x][y2];

					if ( o_value != value ){
						
						did_something	= true;
						
						o_value.setPossible( 
								solve_instance,
								value.getValue(), false, 
								new SudokuPossibilityMetricImpl(
										SudokuPossibilityMetric.MT_ALREADY_IN_SLICE,
										"already in " + (row?"row":"column")));
					}
				}
			}
		}
		
		return( did_something );
	}
	
	public boolean
	update(
		SudokuSolveInstance	solve_instance )
	{
		boolean did_something =	update( solve_instance, model.getValues(), false );
		
		if ( update( solve_instance, model.getReflectedValues(), true )){
			
			did_something = true;
		}
		
		return( did_something );
	}
	
	protected boolean
	update(
		SudokuSolveInstance		solve_instance,	
		SudokuValue[][]			values,
		boolean					row )
	{	
		boolean did_something = false;
		
		for (int x=0;x<model.getWidth();x++){
				
			int[]	possibility_totals = new int[model.getWidth()];
			
			SudokuValue[]	possibility_totals_vals = new SudokuValue[possibility_totals.length];
			
			for (int y=0;y<model.getHeight();y++){
				
				SudokuValue	o_value = values[x][y];

				int	total = 0;
				int	last_possibility = 0;
				
				boolean[] possibilities = o_value.getPossibles( true );
				
				for (int z=0;z<possibilities.length;z++){
					
					if ( possibilities[z] ){
						
						possibility_totals[z]++;
						
						possibility_totals_vals[z]	= o_value;
						
						total++;
						
						last_possibility = z;
					}
				}
				
				if ( total == 1 ){
					
						// System.out.println( "Slicer: " + o_value.getString(true) + " can only be " + ( last_possibility+1 ) + " as it is the only choice");
					
					did_something	= true;
					
					updateValue( 
							solve_instance,
							o_value, last_possibility+1,
							new SudokuValueMetricImpl(
									SudokuValueMetric.MT_SLICE_ONLY_CHOICE,
									(row?"row":"column")+": only choice"));
				}
			}
			
			for (int i=0;i<possibility_totals.length;i++){
				
				if ( possibility_totals[i] == 1 ){
					
						// System.out.println( "Slicer: " + possibility_totals_vals[i].getString(true) + " can only be " + ( i+1 ) + " as no others can be");
					
					did_something	= true;
					
					updateValue( 
							solve_instance,
							possibility_totals_vals[i], i+1, 
							new SudokuValueMetricImpl(
									SudokuValueMetric.MT_SLICE_NO_OTHERS,
									(row?"row":"column")+": no others can be" ));
				}
			}
		}
		
		return( did_something );
	}
	
	public int
	getComplexity()
	{
		return( COMP_SIMPLE );
	}
}
