/*
 * Created on 07-Sep-2005
 * Created by Paul Gardner
 * Copyright (C) 2005 Aelitis, All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * AELITIS, SARL au capital de 30,000 euros
 * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
 *
 */

package org.parg.azureus.plugins.sudoku.model.impl;

import java.util.*;

import org.parg.azureus.plugins.sudoku.model.SudokuModel;
import org.parg.azureus.plugins.sudoku.model.SudokuPossibilityMetric;
import org.parg.azureus.plugins.sudoku.model.SudokuSolveInstance;
import org.parg.azureus.plugins.sudoku.model.SudokuValue;

public class 
SudokuStrategySliceDiceRestriction 
	extends SudokuStrategy
{
	private SudokuModel		model;
		
	private int	DEBUG_X	= -1;//4;
	private int DEBUG_Y	= -1;//2;
	
	protected
	SudokuStrategySliceDiceRestriction(
		SudokuModelImpl		_model )
	{
		super( _model );
		
		model	= _model;
	}
		
	public boolean
	apply(
		SudokuSolveInstance	solve_instance )
	{
		boolean	did_something = false;
				
		SudokuValue[][]	values 				= model.getValues();
		SudokuValue[][]	reflected_values 	= model.getReflectedValues();
		
		for (int x=0;x<model.getWidth();x++){
	
			List	slice1 = new ArrayList();
			List	slice2 = new ArrayList();
			
			for (int y=0;y<model.getHeight();y++){
					
				slice1.add( values[x][y] );
				slice2.add( reflected_values[x][y]);
			}
			
			boolean	debug_col = x==DEBUG_X;
			boolean debug_row = x==DEBUG_Y;
			
			if ( apply( solve_instance, "Col("+x+")",debug_col,slice1, false, x )){
				
				did_something	= true;
			}
			
			if ( apply( solve_instance, "Row("+x+")",debug_row,slice2, true, x )){
				
				did_something = true;
			}
		}
		
		return( did_something );
	}
	
	protected boolean
	apply(
		SudokuSolveInstance	solve_instance,
		String				info,
		boolean				debug,
		List				values,
		boolean				row,
		int					index )
	{
		boolean did_something = false;
		
		try{

				// for each row/slice look for numbers that can only appear in a given
				// dice
						
			int	size = model.getSize();
			
			int[]	hits = new int[size*size];
			
			Arrays.fill( hits, -1 );
			
			for (int i=0;i<values.size();i++){
				
				SudokuValue	value = (SudokuValue)values.get(i);
			
				if ( value.hasValue()){
					
					continue;
				}
				
				int	dice = i/size;
				
				boolean[]	poss = value.getPossibles( false );
				
				for (int j=0;j<poss.length;j++){
					
					if ( poss[j] ){
			
						if ( hits[j] == -1 ){
							
							hits[j] = dice;
							
						}else if ( hits[j] != dice ){
							
							hits[j]	= -2;
						}
					}
				}
			}
			
			for (int i=0;i<hits.length;i++){
					
				if ( hits[i] >= 0 ){
						
					// System.out.println((row?"row":"col") + " " + ( index+1) + ": " + ( i + 1 ) + " -> dice " + hits[i] );
					
					List	dice_values = row?getDice( hits[i], index/size ):getDice( index/size,  hits[i] );
					
					for (int j=0;j<dice_values.size();j++){
						
						SudokuValueImpl	dice_value = (SudokuValueImpl)dice_values.get(j);
	
						if ( !values.contains( dice_value )){
							
							boolean[]	dice_poss = dice_value.getPossibles( false );

							if ( dice_poss[i] ){
								
								did_something = true;
								
									// System.out.println( "Setting " + dice_value.getString(0) + ", poss " + (i+1) + " false" );								
								
								dice_value.setPossible( 
										solve_instance,
										i+1, false,
										new SudokuPossibilityMetricImpl(
												SudokuPossibilityMetric.MT_SLICE_DICE_RESTRICTION,
												"slice dice restriction: value " + (i+1) + " must occur in " + (row?"row":"col") + " " + ( index+1 )));
								
							}
						}
					}
				}
			}
			
			return( did_something );
			
		}catch( Throwable e ){
			
			e.printStackTrace();
			
			return( false );
		}
	}

	protected List
	getDice(
		int		row,
		int		col )
	{
		int	model_size = model.getSize();
		
		SudokuValue[][]	values 				= model.getValues();

		List	dice = new ArrayList();
				
		for (int xc=model_size*row;xc<(row+1)*model_size;xc++){
					
			for (int yc=model_size*col;yc<(col+1)*model_size;yc++){

				SudokuValue	value = values[xc][yc];
					
				if ( !value.hasValue()){
					
					dice.add( value );
				}
			}
		}
				
		return( dice );
	}
	
	public boolean
	update(
		SudokuSolveInstance	solve_instance )
	{	 
		return( false );
	}
	
	public int
	getComplexity()
	{
		return( COMP_COMPLEX );
	}
}
