/*
 * Created on 07-Sep-2005
 * Created by Paul Gardner
 * Copyright (C) 2005 Aelitis, All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * AELITIS, SARL au capital de 30,000 euros
 * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
 *
 */

package org.parg.azureus.plugins.sudoku.model.impl;

import java.util.*;

import org.parg.azureus.plugins.sudoku.model.SudokuModel;
import org.parg.azureus.plugins.sudoku.model.SudokuPossibilityMetric;
import org.parg.azureus.plugins.sudoku.model.SudokuSolveInstance;
import org.parg.azureus.plugins.sudoku.model.SudokuValue;

public class 
SudokuStrategyTupleCycles 
	extends SudokuStrategy
{
	private SudokuModel		model;
		
	private int	DEBUG_X	= -1;//4;
	private int DEBUG_Y	= -1;//2;
	
	protected
	SudokuStrategyTupleCycles(
		SudokuModelImpl		_model )
	{
		super( _model );
		
		model	= _model;
	}
		
	public boolean
	apply(
		SudokuSolveInstance	solve_instance )
	{
		boolean	did_something = false;
		
		int	model_size = model.getSize();
		
		SudokuValue[][]	values 				= model.getValues();
		SudokuValue[][]	reflected_values 	= model.getReflectedValues();

		for (int x=0;x<model.getWidth();x+=model_size){
			
			for (int y=0;y<model.getHeight();y+=model_size){
	
				List	dice = new ArrayList();
				
				for (int xc=x;xc<x+model_size;xc++){
					
					for (int yc=y;yc<y+model_size;yc++){

						SudokuValue	value = values[xc][yc];
						
						dice.add( value );
					}
				}
				
				boolean	debug = 
							DEBUG_X >= 0 &&
							DEBUG_Y >= 0 &&
							x/model_size==DEBUG_X/model_size 
							&& y/model_size==DEBUG_Y/model_size;
				
				if ( apply( solve_instance, "Dice("+(x/model_size)+"/"+(y/model_size)+")", debug, dice, true, false )){
					
					did_something	= true;
				}
			}
		}
		
		for (int x=0;x<model.getWidth();x++){
	
			List	slice1 = new ArrayList();
			List	slice2 = new ArrayList();
			
			for (int y=0;y<model.getHeight();y++){
					
				slice1.add( values[x][y] );
				slice2.add( reflected_values[x][y]);
			}
			
			boolean	debug_col = x==DEBUG_X;
			boolean debug_row = x==DEBUG_Y;
			
			if ( apply( solve_instance, "Col("+x+")",debug_col,slice1,false, false )){
				
				did_something	= true;
			}
			
			if ( apply( solve_instance, "Row("+x+")",debug_row,slice2,false, true )){
				
				did_something = true;
			}
		}
		
		return( did_something );
	}
	
	protected boolean
	apply(
		SudokuSolveInstance	solve_instance,
		String				info,
		boolean				debug,
		List				values,
		boolean				chunk, 
		boolean				row )
	{
		boolean did_something = false;
		
		try{
		if ( debug ){
			String	entry_str = "";
			
			for (int i=0;i<values.size();i++){
				SudokuValue	value = (SudokuValue)values.get(i);
				if ( !value.hasValue()){
					entry_str += value.getString(1);
				}
			}
			System.out.println( info + ": Tuples: " + entry_str );
		}
		
		int	num_possible = model.getSize()*model.getSize();
		
		List[]	groups = new List[num_possible+1];
		
		for (int i=0;i<values.size();i++){
			
			SudokuValue	value = (SudokuValue)values.get(i);
			
			if ( value.hasValue()){
				
				continue;
			}
			
			boolean[]	possibles = value.getPossibles( false );
			
			int	num = 0;
			
			for (int j=0;j<possibles.length;j++){
				
				if ( possibles[j] ){
			
					num++;
				}
			}
			
			if ( groups[num] == null ){
				
				groups[num] = new ArrayList(num_possible);
			}
			
			groups[num].add( value );	
		}
		
		for (int i=2;i<groups.length;i++){
			
			List	entries = groups[i];
			
				// for size 2, look for runs of 2 (a,b) (a,b), runs of 3 (a,b) (b,c) (c,d) etc
				// for size 3, look for runs of 3 (a,b,c) (a,b,c) (a,b,c), runs of 4 (a,b,c) (b,c,d) (c,d,a) (a,b,d) etc
			
			if ( entries == null ){
				
				continue;
			}
			
			for (int j=i;j<=entries.size();j++){
				
					// get all permutations of j
				
				List	perms = permute( entries, j );
				
				for (int k=0;k<perms.size();k++){
					
					List	perm = (List)perms.get(k);
										
					int[]	possibles_count = new int[num_possible];
					
					for (int p=0;p<perm.size();p++){
						
						SudokuValue	value = (SudokuValue)perm.get(p);
	
						boolean[]	possibles = value.getPossibles( false );
	
						for (int z=0;z<possibles.length;z++){
						                                     
							if ( possibles[z]){
								
								possibles_count[z]++;
							}
						}
					}
					
					int	hits = 0;
					
					String	tuple_str = "";
					
					for (int p=0;p<possibles_count.length;p++){
						
						if ( possibles_count[p] == i ){
							
							tuple_str += (tuple_str.length()==0?"":",") + (p+1);
							
							hits++;
						}
					}
				
					boolean	match = hits == j;
				
					//System.out.println( "        perm:" + j + ":->" + str + " - match = " + match );
					
						// a match means that we remove these possibilities from all other
						// vacant entries in the set
					
					if ( match ){
						
						for (int v=0;v<values.size();v++){
							
							SudokuValueImpl	value = (SudokuValueImpl)values.get(v);
							
							if ( (!value.hasValue()) && !perm.contains( value )){
								
								for (int p=0;p<possibles_count.length;p++){
									
									if ( possibles_count[p] == i ){
								
										//System.out.println( "setting " + value.getString(true) + "/" + ( p+1 ) + "->false");
										
										did_something	= true;
										
										value.setPossible( 
												solve_instance,
												p+1, 
												false, 
												new SudokuPossibilityMetricImpl(
													chunk?SudokuPossibilityMetric.MT_CHUNK_TUPLE_EXCLUSION:SudokuPossibilityMetric.MT_SLICE_TUPLE_EXCLUSION,
													(chunk?"chunk":(row?"row":"column")) + ": tuple cycle exclusion (" + tuple_str + ":" + j + ")",
													new int[]{ j, hits }));
									}
								}
							}
						}
					}
				}
			}
		}
		
		if ( debug ){
			String	entry_str = "";
			
			for (int i=0;i<values.size();i++){
				SudokuValue	value = (SudokuValue)values.get(i);
				if ( !value.hasValue()){
					entry_str += value.getString(1);
				}
			}
			System.out.println( info + "  P1 : Tuples: " + entry_str );
		}
			// next up are buried sequences - that is cells that have extra possibilities that
			// are actually impossible.
			// consider (1,2,3) (1,2,4) - this contains (1,2) and (1,2), hence the (3) and (4)
			// possibilities are irrelevant.
			// however, note that for these reductions to be valid the possibilities in the
			// sequence (in this case 1,2) must NOT appear in any other values outside of the 
			// sequence. e.g. if we also had (1,2,5) then we could satisfy the set with (1,4,2)
		
			// for each number count the number of places it occurs
	
		List[]	numbers = new List[num_possible];

		for (int i=0;i<values.size();i++){
			
			SudokuValue	value = (SudokuValue)values.get(i);
			
			if ( value.hasValue()){
				
				continue;
			}
			
			boolean[]	possibles = value.getPossibles( false );
						
			for (int j=0;j<possibles.length;j++){
				
				if ( possibles[j] ){
			
					if ( numbers[j] == null ){
						
						numbers[j] = new ArrayList();
					}
					
					numbers[j].add( value );
				}
			}
		}
		
			// remove any values that can only be one possible value from consideration

		for (int i=0;i<numbers.length;i++){
			
			if ( numbers[i] == null ){
				
				continue;
			}
			
			if ( numbers[i].size() == 1 ){
				
				SudokuValue	to_remove = (SudokuValue)numbers[i].get(0);
				
				for (int j=0;j<numbers.length;j++){
					
					if ( numbers[j] != null ){
						
						numbers[j].remove( to_remove );
					}
				}
				numbers[i] = null;
			}
		}
			// now group by number of occurrences
		
		List[]	number_groups = new List[num_possible+1];
		
		for (int i=0;i<numbers.length;i++){
			
			List	nums = numbers[i];
			
			if ( nums != null ){
				
				int	count = nums.size();
				
				if ( number_groups[count] == null ){
					
					number_groups[count] = new ArrayList();
				}
				
				number_groups[count].add( new Integer(i));
			}
		}


		if ( debug ){
			for (int i=0;i<number_groups.length;i++){
				List	l = number_groups[i];
				if ( l == null ){
					continue;
				}
				String	s = "";
				for (int j=0;j<l.size();j++){
					s += (j==0?"":",")+(((Integer)l.get(j)).intValue()+1);
				}
				System.out.println( "ng:" + i + " -> " + s );
			}
		}
		
			// for each number group look for potential tuple cycles. For example, if we have
			// 2 numbers that occur three times (from, say, (1,2,4) (1,3,4) (2,3,5) then we'll get
			// number groups 1{} 2{1,2,3,4} 3{}. We permute {1,2,3,4} as usual to get
			// {1,2} {1,3} {1,4} {2,3} {2,4} {3,4}
		
		
		for (int i=2;i<number_groups.length-1;i++){	// -1 as no point in considering max len
			
			List	number_group = number_groups[i];
			
			if ( number_group == null ){
				
				continue;
			}
			
			for (int cycle_length=i;cycle_length<=number_group.size();cycle_length++){
	
				List	perms = permute( number_group, cycle_length );
			
				for (int j=0;j<perms.size();j++){
					
					List	perm = (List)perms.get(j);
					
						// count up the number of values involved in this cycle. If it is the same	
						// as the number group size then we've got a set of cells participating in 
						// a cycle via a subset of the possibilities and can remove the extra
						// irrelevant possibilities
					
					Set	unique_values = new HashSet();
					
					for (int k=0;k<perm.size();k++){
						
						Integer	poss = (Integer)perm.get(k);
						
						List	nums = numbers[poss.intValue()];
						
						for ( int n=0;n<nums.size();n++){
							
							unique_values.add(nums.get(n));
						}
					}
					
					if ( unique_values.size() == cycle_length ){
						
						String	perm_str = "";
						for (int k=0;k<perm.size();k++){
							
							Integer	poss = (Integer)perm.get(k);
							perm_str += (k==0?"":",")+(((Integer)poss).intValue()+1);
						}
						
						if ( debug){
							String	ng_str = "";
							for (int k=0;k<number_group.size();k++){
								ng_str += (k==0?"":",")+number_group.get(k);
							}
												
							System.out.println( "hit: ng=" + i + "(" + ng_str + ") cycle = " + cycle_length + ", nums=" + perm_str );
						}
						
							// remove excess values
						
						Iterator it = unique_values.iterator();
							
						while (it.hasNext()){
							
							SudokuValueImpl	value = (SudokuValueImpl)it.next();
							
							boolean[]	possibilities = value.getPossibles( false );
							
							for (int p=0;p<possibilities.length;p++){
													
								if ( possibilities[p] && !perm.contains( new Integer(p))){
									
									did_something = true;
									
									value.setPossible( 
											solve_instance,
											p+1, false,
											new SudokuPossibilityMetricImpl(
													chunk?SudokuPossibilityMetric.MT_CHUNK_TUPLE_REDUCTION:SudokuPossibilityMetric.MT_SLICE_TUPLE_REDUCTION,
													(chunk?"chunk":(row?"row":"column")) + ": tuple cycle reduction (" + perm_str + ":" + i + ")",
													new int[]{ i, cycle_length }));
													
								}
							}
						}
					}
				}
			}
		}
		
		
		if ( debug ){
			String	entry_str = "";
			
			for (int i=0;i<values.size();i++){
				SudokuValue	value = (SudokuValue)values.get(i);
				if ( !value.hasValue()){
					entry_str += value.getString(1);
				}
			}
			System.out.println( info + "  P2 : Tuples: " + entry_str );
		}
		
		}catch( Throwable e ){
			e.printStackTrace();
		}
		
		return( did_something );
	}

	protected List
	permute(
		List	_values,
		int		num )
	{
		List	res = new ArrayList();
		
		if ( _values.size() < num ){
			
			return( res );
			
		}else if ( _values.size() == num ){
			
			res.add( _values );
			
			return( res );
		}

		int[]		pointers = new int[num];
		
		Object[]	values = _values.toArray();
		
		for (int i=0;i<num;i++){
			
			pointers[i] = i;
		}
		
		while( true ){
			
			List entry = new ArrayList(num);
			
			for (int i=0;i<pointers.length;i++){
				
				entry.add( values[pointers[i]]);
			}
			
			res.add( entry );
			
			if ( !updatePointers( pointers, num-1, values.length )){
				
				break;
			}
		}
		
		return( res );
	}
	
	protected boolean
	updatePointers(
		int[]		pointers,
		int			pointer,
		int			num )
	{
		if ( ++pointers[pointer] < num ){
			
			return( true );
		}
	
		if ( pointer == 0 ){
			
			return( false );
		}
		
		if ( updatePointers( pointers, pointer-1, num-1 )){
	
			pointers[pointer] = pointers[pointer-1]+1;
			
			return( true );
		}
		
		return( false );
	}
	
	public boolean
	update(
		SudokuSolveInstance	solve_instance )
	{	
		return( false );
	}
	
	public int
	getComplexity()
	{
		return( COMP_COMPLEX );
	}
}
