/*
 * Created on 07-Sep-2005
 * Created by Paul Gardner
 * Copyright (C) 2005 Aelitis, All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * AELITIS, SARL au capital de 30,000 euros
 * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
 *
 */

package org.parg.azureus.plugins.sudoku.model.impl;

import java.util.*;

import org.parg.azureus.plugins.sudoku.model.SudokuModel;
import org.parg.azureus.plugins.sudoku.model.SudokuPossibilityMetric;
import org.parg.azureus.plugins.sudoku.model.SudokuSolveInstance;
import org.parg.azureus.plugins.sudoku.model.SudokuValue;

public class 
SudokuStrategyXWing 
	extends SudokuStrategy
{
	private SudokuModel		model;
		
	private int	DEBUG_X	= -1;//4;
	private int DEBUG_Y	= -1;//2;
	
	protected
	SudokuStrategyXWing(
		SudokuModelImpl		_model )
	{
		super( _model );
		
		model	= _model;
	}
		
	public boolean
	apply(
		SudokuSolveInstance	solve_instance )
	{
			// look for rows with, say, 2 occurrences of possibility X
			// then see if there are 2 such rows with X in exactly the same place
			// if this is the case then any other X in the columns with the X in can't occur
			// 
			// This also applies to 3 occurrences of X that occur in 3 rows etc
			// (and obviously to reflection too)
				
		boolean	did_something = false;
				
		SudokuValue[][]	values 				= model.getValues();
		SudokuValue[][]	reflected_values 	= model.getReflectedValues();
		
		if ( apply( solve_instance, values, false )){
			
			did_something = true;
		}
		
		if ( apply( solve_instance, reflected_values, true )){
			
			did_something = true;
		}
		
		return( did_something );
	}
	
	protected boolean
	apply(
		SudokuSolveInstance		solve_instance,
		SudokuValue[][]			values,
		boolean					is_row )
	{
		int	width = model.getWidth();
		
		BitSet[][]	bit_sets = new BitSet[width][width];
		
		for (int x=0;x<width;x++){
	
			SudokuValue[]	row = values[x];
			
			for (int y=0;y<row.length;y++){
				
				SudokuValue	value = row[y];
				
				if ( value.hasValue()){
					
					continue;
				}
								
				boolean[]	poss = value.getPossibles( false );
					
				for (int z=0;z<poss.length;z++){
					
					if ( poss[z] ){
						
						if ( bit_sets[x][z] == null ){
							
							bit_sets[x][z] = new BitSet();
						}
						
						bit_sets[x][z].set( y );
					}
				}
			}
		}		
	
		for (int x1=0;x1<width;x1++){

			for (int s=0;s<width;s++ ){
				
				BitSet	set1 = bit_sets[x1][s];
				
				if ( set1 == null ){
				
					continue;
				}
				
				int	num = set1.cardinality();
				
				if ( num == 1 || num == width ){
					
					continue;
				}
				
					// set 1 is col map  of poss in row x set to s
								
				List	matches = null;
				
					// look for 'num-1' other rows with same mapping
				
				for ( int x2=x1+1;x2<width;x2++){
					
					BitSet	set2 = bit_sets[x2][s];
					
					if ( set2 == null ){
					
						continue;
					}
					
					if ( set2.equals( set1 )){
						
						if ( matches == null ){
							
							matches = new ArrayList();
							
							matches.add( new Integer( x1));
						}
						
						matches.add( new Integer( x2 ));
					}
				}
				
				if ( matches != null && matches.size() == num ){
					
					// System.out.println( "XWing match on " + (is_row?"row":"col") + " " + ( x1+1) + ", num " + (s+1) + ", count = " + num );
					
						// remove number from other rows
					
					for (int x2=0;x2<width;x2++){
						
						if ( !matches.contains( new Integer( x2 ))){
							
							for (int c=0;c<width;c++){
								
								if ( set1.get(c)){
									
									SudokuValueImpl		v = (SudokuValueImpl)values[x2][c];

									if ( v.hasValue()){
										
										continue;
									}
									
									if ( v.getPossibles(false)[s]){
										
										// System.out.println( "    clearing entry " + (x2+1) + "/" + ( c+1 ));
										
										String	rows_str = "";
										
										for (int z=0;z<matches.size();z++){
												
											rows_str += (z==0?"":",") + (((Integer)matches.get(z)).intValue() + 1 );
										}
										
										v.setPossible( 
												solve_instance,
												s+1, false,
												new SudokuPossibilityMetricImpl(
														SudokuPossibilityMetric.MT_X_WING,
														"x-wing: value " + (s+1) + " "+ (is_row?"rows":"cols") + " " + rows_str + ", size " + num ));

									}
								}
							}
						}
					}
				}
			}
		}
		
		return( false );
	}
	

	
	public boolean
	update(
		SudokuSolveInstance	solve_instance )
	{	
		return( false );
	}
	
	public int
	getComplexity()
	{
		return( COMP_XWING );
	}
}
