/*
 * Created on 04-Sep-2005
 * Created by Paul Gardner
 * Copyright (C) 2005 Aelitis, All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * AELITIS, SARL au capital de 30,000 euros
 * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
 *
 */

package org.parg.azureus.plugins.sudoku.model.impl;

import java.util.*;

import org.parg.azureus.plugins.sudoku.model.*;

public class 
SudokuValueImpl
	implements SudokuValue
{
	private List		listeners	= new ArrayList();
	
	private	SudokuModelImpl	model;
	
	private int			x_pos;
	private int			y_pos;


		// ***** change these you need to fix up export/clone
	
	private int						value		= -1;
	private SudokuValueMetric		value_reason;
	private boolean					explicit	= false;
	private boolean					core		= false;
	private boolean					hint		= false;
	private int						hint_value	= -1;
	
	private boolean[]					possibles;
	private SudokuPossibilityMetric[]	possibles_reasons;
		
	private int[]						possible_overrides;

	private boolean[]					hint_possibles;
	private SudokuPossibilityMetric[]	hint_possibles_reasons;
	
		// ******** 
	
	protected
	SudokuValueImpl(
		SudokuModelImpl	_model,
		int				_x_pos,
		int				_y_pos,
		int				_size )
	{
		model	= _model;
		x_pos	= _x_pos;
		y_pos	= _y_pos;
		
		possibles			= new boolean[_size];
		possible_overrides	= new int[_size];
		
		possibles_reasons	= new SudokuPossibilityMetric[_size];
		
		setAllPossible();
	}
	
	public boolean
	hasValue()
	{
		return( value != -1 );
	}
	
	public int
	getValue()
	{
		return( value );
	}
	
	public SudokuValueMetric
	getValueMetric()
	{
		return( value_reason );
	}
	
	public void
	setValue(
		int						value,
		boolean					explicit )
	{
		setValue( 
				null,
				value, 
				explicit, 
				new SudokuValueMetricImpl(
						SudokuValueMetric.MT_ENTERED,
						"Entered" ));
	}
	
	public void
	setValue(
		SudokuSolveInstance	solve_instance,
		int					_value,
		boolean				_explicit,
		SudokuValueMetric	_reason )
	{
		if ( _value == -1 ){
			
			_explicit = false;
			
		}else{
			
			if ( !_explicit ){
				
				if ( hasValue() && value != _value ){
					
						// trying to override a value here, existing value must be wrong. flag
						// by marking the new value as the only value one
					
					for (int i=0;i<possibles.length;i++){
						
						possibles[i] = (i==(_value-1));
					}
					
					informPossibilityChanged( solve_instance, _value+1, true );
					
					return;
				}
			}
		}
		
		if ( value != _value ){
			
			value			= _value;
			value_reason	= _reason;
			explicit		= _explicit;
			
			if ( value == -1 ){
				
				hint			= false;
			}
						
			informValueChanged( solve_instance );
			
		}else if ( _explicit && !explicit ){
			
			explicit	= _explicit;

			informValueChanged(solve_instance);
		}
	}
	
	public boolean
	isExplicit()
	{
		return( explicit );
	}
	
	public boolean
	isCore()
	{
		return( core );
	}
	
	public void
	setCore(
		boolean	c )
	{
		core	= c;
	}
	
	public boolean
	isHint()
	{
		return( hint );
	}
	
	protected int
	getHintValue()
	{
		return( hint_value );
	}
	
	protected void
	setHint(
		boolean	c )
	{
		//boolean	was_explicit = explicit;
		
		//explicit	= true;
		hint		= c;
		hint_value	= value;
		
		informOtherChanged();
	}
	
	public void
	clearHint()
	{
		model.removeHint( this );
	}
	
	public int
	getX()
	{
		return( x_pos );
	}
	
	public int
	getY()
	{
		return( y_pos );
	}
		
	public boolean[]
	getPossibles(
		boolean	strict )
	{
		if ( strict && hasValue()){
			
			boolean[]	temp = new boolean[possibles.length];
			
			temp[value-1] = true;
			
			return( temp );
		}else{
			
			return( possibles );
		}
	}
	
	public void
	setPossible(
		SudokuSolveInstance		solve_instance,
		int						value,
		boolean					possible,
		SudokuPossibilityMetric	reason )
	{
		/*
		int	ot = possible_overrides[value-1];
		
		if ( ot == OT_FALSE){
			
			possible	= false;
			
		}else if ( ot == OT_TRUE ){
			
			possible	= true;
		}
		*/
		
		if ( possibles[value-1] != possible ){
			
			possibles[value-1] = possible;

			possibles_reasons[value-1] = reason;

			informPossibilityChanged( solve_instance, value, possible );
		}
	}
	
	public boolean
	isPossible(
		int		value )
	{
		return( possibles[value-1]);
	}
	
	protected void
	setAllPossible()
	{	
		for (int i=0;i<possibles.length;i++){
			
			//int	ot = possible_overrides[i];
			
			possibles[i]			= true; // ot==OT_TRUE||ot==OT_NONE;
			possibles_reasons[i]	= null;
		}	
	}
	
	public void
	setPossibleOverride(
		int		value,
		int		ot  )
	{
		possible_overrides[value-1] = ot;
			
		informOtherChanged();
		
		// possibles[value-1] = ot==OT_TRUE||ot==OT_NONE;
			
		// informValueChanged(new SudokuSolveInstanceImpl(false,99));
	}
	
	public int[]
	getPossibleOverrides()
	{
		return( possible_overrides );
	}
	
	protected void
	setPossibleOverrides(
		int[]		po )
	{
		possible_overrides = po;
	}
		
	public SudokuPossibilityMetric[]
	getPossibleReasons()
	{
		return( possibles_reasons );
	}
	
	public Object
	exportHints()
	{
		hint_possibles			= (boolean[])possibles.clone();
		hint_possibles_reasons	= new SudokuPossibilityMetric[possibles_reasons.length];
		
		for (int i=0;i<hint_possibles_reasons.length;i++){
			
			if ( possibles_reasons[i] != null ){
			
				hint_possibles_reasons[i] = (SudokuPossibilityMetric)possibles_reasons[i].clone();
			}
		}

		return( new Object[]{ hint_possibles, hint_possibles_reasons });
	}
	
	public void
	importHints(
		Object	o )
	{
		Object[]	hints = (Object[])o;
		
		hint_possibles			= (boolean[])hints[0];
		hint_possibles_reasons	= (SudokuPossibilityMetric[])hints[1];
		
		informPossibilityChanged( null, -1, false );
	}
	
	public boolean[]
	getHintPossibles()
	{
		return( hint_possibles );
	}
	
	public SudokuPossibilityMetric[]
	getHintPossibleReasons()
	{
		return( hint_possibles_reasons );
	}
	
	public void
	reset()
	{
		reset( false );
	}

	public void
	reset(
		boolean	reset_non_core )
	{
		setAllPossible();
				
		if ( !explicit ){
			
			value 			= -1;
			value_reason	= null;
			
		}else if ( reset_non_core && !core ){
			
			value = -1;
		}
		
		if ( reset_non_core ){
			
			hint			= false;
			value_reason	= null;
			
			hint_possibles			= null;
			hint_possibles_reasons	= null;
			
			Arrays.fill( possible_overrides, OT_NONE );
		}
		
		if ( !explicit ){
			informValueChanged( null ); // !explicit);
		}
	}
	
	public void
	clear()
	{
		setAllPossible();
							
		value 			= -1;
		value_reason	= null;
		
		explicit = false;
		
		core	= false;
		
		hint	= false;
		
		hint_possibles			= null;
		hint_possibles_reasons	= null;

		Arrays.fill( possible_overrides, OT_NONE );

		informValueChanged( null );//!explicit);
	}
	
	public boolean
	isValid()
	{
		if ( hasValue()){
			
			return( isPossible( value ));
			
		}else{
			
			for (int i=0;i<possibles.length;i++){
				
				if ( possibles[i] ){
					
					return( true );
				}	
			}
			
			return( false );
		}
	}
	
	public SudokuModel
	getModel()
	{
		return( model );
	}
	
	protected void
	informValueChanged(
		SudokuSolveInstance	solve_instance )
	{
		if ( model.disableNotifications()){
			
			return;
		}
		
		if ( solve_instance == null ){
			
			solve_instance = new SudokuSolveInstanceImpl(false,model.getHeight()*model.getWidth());
		}
	
		for (int i=0;i<listeners.size();i++){
			
			((SudokuValueListener)listeners.get(i)).valueChanged( solve_instance, this );
		}
	}
	
	protected void
	informPossibilityChanged(
		SudokuSolveInstance	solve_instance,
		int					val,
		boolean				b )
	{
		if ( model.disableNotifications()){
			
			return;
		}
		
		if ( solve_instance == null ){
			
			solve_instance = new SudokuSolveInstanceImpl(false,model.getHeight()*model.getWidth());
		}
		
		for (int i=0;i<listeners.size();i++){
			
			((SudokuValueListener)listeners.get(i)).possibilityChanged( solve_instance, this, val, b );
		}
	}
	
	protected void
	informOtherChanged()
	{
		if ( model.disableNotifications()){
			
			return;
		}
		
		for (int i=0;i<listeners.size();i++){
			
			((SudokuValueListener)listeners.get(i)).otherChanged( this );
		}
	}
	
	public void
	addListener(
		SudokuValueListener		listener )
	{
		listeners.add( listener );
	}
	
	public void
	removeListener(
		SudokuValueListener		listener )
	{
		listeners.remove( listener );
	}
	
	public Object
	clone()
	{
			// shallow copy simple fields
		
		try{
			SudokuValueImpl	clone = (SudokuValueImpl)super.clone();
			
				// deep copy as required
			
			clone.possibles				= (boolean[])possibles.clone();
			
			if ( value_reason != null ){
				
				clone.value_reason			= (SudokuValueMetric)value_reason.clone();
			}
			
			clone.possibles_reasons		= new SudokuPossibilityMetric[possibles_reasons.length];
			
			for (int i=0;i<possibles_reasons.length;i++){
				
				if ( possibles_reasons[i] != null ){
					
					clone.possibles_reasons[i] = (SudokuPossibilityMetric)possibles_reasons[i].clone();
				}
			}
			
			return( clone );
			
		}catch( CloneNotSupportedException e ){
			
			e.printStackTrace();
			
			return( null );
		}
	}
	
	protected SudokuValueImpl
	exportAll()
	{
		return((SudokuValueImpl)clone());
	}
	
	protected void
	importAll(
			SudokuValueImpl	data )
	{
		SudokuValueImpl	v = (SudokuValueImpl)((SudokuValueImpl)data).clone();
		
		value				= v.value;
		value_reason		= v.value_reason;
		explicit			= v.explicit;
		core				= v.core;
		hint				= v.hint;
		hint_value			= v.hint_value;
		
		possibles 				= v.possibles;
		possibles_reasons		= v.possibles_reasons;
			
		possible_overrides		= v.possible_overrides;

		hint_possibles			= v.hint_possibles;
		hint_possibles_reasons	= v.hint_possibles_reasons;
	}
	
	public String
	getString(
		int		level )
	{
		if ( level == 0 ){
			
			return( "(" + (x_pos+1) + "," + (y_pos+1) + ")" );
			
		}else if ( level == 1 ){
			
			String	poss = "";
			
			for (int i=0;i<possibles.length;i++){
				if ( possibles[i]){
					poss += (poss.length()==0?"":",")+(i+1);
				}
			}
			return( "(" +(x_pos+1)+"," + (y_pos+1)+":"+poss+")");
			
		}else{
		
			return( "(" + (x_pos+1) + "," + (y_pos+1) + ": = " + value + "/" + explicit);
		}
	}
}
