/*
 * Created on 04-Sep-2005
 * Created by Paul Gardner
 * Copyright (C) 2005 Aelitis, All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * AELITIS, SARL au capital de 30,000 euros
 * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
 *
 */

package org.parg.azureus.plugins.sudoku.ui.swing;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.print.*;


import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextArea;
import javax.swing.RepaintManager;
import javax.swing.SwingUtilities;

import org.parg.azureus.plugins.sudoku.model.*;
import org.parg.azureus.plugins.sudoku.util.Semaphore;


public class 
SudokuUIMain 
	implements SudokuModelListener
{
	private SudokuModel			model;
	private SudokuUIAdapter		adapter;
	
	private SudokuUIValue[][]	ui_values;
	
	private JPanel	main;
	
	private JLabel	status;
	
	private JMenuItem	level_easy;
	private JMenuItem	level_medium;
	private JMenuItem	level_hard;
	private JMenuItem	level_ultimate;
	
	private JMenuItem	symmetry_none;
	private JMenuItem	symmetry_horizontal;
	private JMenuItem	symmetry_vertical;
	private JMenuItem	symmetry_square;

	
	
	private boolean	auto_solve			= false;
	
	private boolean show_possibilities	= false;
	private boolean show_errors			= false;
	
	public 
	SudokuUIMain(
		SudokuModel		_model,
		SudokuUIAdapter	_adapter )
	{
		model	= _model;
		adapter	= _adapter;
		
		model.addListener( this );
		
		SudokuValue[][]	values = model.getValues();

		main = new JPanel();
		
		main.setLayout( new GridBagLayout());
				
		int	row_pos = 0;
		
		JMenuBar	menu_bar = new JMenuBar();
		
		JMenu	file_menu = new JMenu( "File" );
		
		file_menu.setMnemonic( 'F' );
		
		JMenuItem	file_generate = new JMenuItem( "Generate" );
		
		file_generate.addActionListener(
				new ActionListener()
				{
					public void
					actionPerformed(
						ActionEvent	ev )
					{
						generate();
					}
				});
		
		JMenuItem	file_reset = new JMenuItem( "Reset" );
		
		file_reset.addActionListener(
				new ActionListener()
				{
					public void
					actionPerformed(
						ActionEvent	ev )
					{
						model.reset();
					}
				});
		
		JMenuItem	file_open = new JMenuItem( "Load" );
		
		file_open.addActionListener(
				new ActionListener()
				{
					public void
					actionPerformed(
						ActionEvent	ev )
					{
						JFileChooser	chooser = new JFileChooser();
								
						int returnVal = chooser.showOpenDialog(main);
						
						if ( returnVal == JFileChooser.APPROVE_OPTION ){
						
							try{
								model.importValues( chooser.getSelectedFile());
								
							}catch( Throwable e ){
								
								e.printStackTrace();
								
								JOptionPane.showMessageDialog(
									main,
									"Load failed: " + e.getMessage(),
									"Load failed",
									JOptionPane.ERROR_MESSAGE );
							}
						}						
					}
				});
		
		JMenuItem	file_save = new JMenuItem( "Save" );
		
		file_save.addActionListener(
				new ActionListener()
				{
					public void
					actionPerformed(
						ActionEvent	ev )
					{
						JFileChooser	chooser = new JFileChooser();
						
						int returnVal = chooser.showSaveDialog(main);
						
						if ( returnVal == JFileChooser.APPROVE_OPTION ){
						
							try{
								model.exportValues( chooser.getSelectedFile());
								
							}catch( Throwable e ){
								
								JOptionPane.showMessageDialog(
									main,
									"Save failed: " + e.getMessage(),
									"Save failed",
									JOptionPane.ERROR_MESSAGE );
							}
						}							
					}
				});
		
		final JPanel	game_panel = new JPanel( new GridBagLayout());
		
		JMenuItem	file_print = new JMenuItem( "Print" );
		
		file_print.addActionListener(
				new ActionListener()
				{
					public void
					actionPerformed(
						ActionEvent	ev )
					{
						try{
							PrinterJob printJob = PrinterJob.getPrinterJob();
							
						    printJob.setPrintable(
						    	new Printable()
						    	{
						    		public int 
						    		print(
						    			Graphics 		g, 
						    			PageFormat 		pageFormat, 
						    			int 			pageIndex) 
						    		{			    			
						    			if (pageIndex > 0){
						    				
						    				return(NO_SUCH_PAGE);
						    				
						    			}else{
						    				
						    				Graphics2D g2d = (Graphics2D)g;
						    				
						    			    g2d.translate(pageFormat.getImageableX(), pageFormat.getImageableY());
						    			    
						    			    RepaintManager currentManager = RepaintManager.currentManager( game_panel );
						    			    
						    			    currentManager.setDoubleBufferingEnabled(false);
		    			    
						    			    game_panel.paint(g2d);
						    			    
						    			    currentManager.setDoubleBufferingEnabled(true);
						    			    
						    			    return(PAGE_EXISTS);
						    			}
						    		}
						    	});
						    
						    if ( printJob.printDialog()){
						      
						        printJob.print();
	
						    }	
						}catch( Throwable e ){
							
							JOptionPane.showMessageDialog(
								main,
								"Print failed: " + e.getMessage(),
								"Print failed",
								JOptionPane.ERROR_MESSAGE );
						}	
					}
				});
		
		
		JMenuItem	file_clear = new JMenuItem( "Clear" );
		
		file_clear.addActionListener(
				new ActionListener()
				{
					public void
					actionPerformed(
						ActionEvent	ev )
					{
						model.clear();
					}
				});
		
		JMenuItem	file_exit = new JMenuItem( "Exit" );
		
		file_exit.addActionListener(
				new ActionListener()
				{
					public void
					actionPerformed(
						ActionEvent	ev )
					{
						adapter.closed();
					}
				});
		
		file_menu.add( file_open );
		file_menu.add( file_save );
		
		file_menu.add( new JSeparator());
		
		file_menu.add( file_print );

		file_menu.add( new JSeparator());

		file_menu.add( file_generate );
		file_menu.add( file_reset );
		
		file_menu.add( new JSeparator());
		
		file_menu.add( file_clear );
				
		file_menu.add( new JSeparator());

		file_menu.add( file_exit );

		menu_bar.add( file_menu );
		
			// level
		
		JMenu	level_menu = new JMenu( "Level" );
		
		level_menu.setMnemonic( 'L' );
		
		level_easy 		= new JCheckBoxMenuItem( "Easy", true );
		level_medium 	= new JCheckBoxMenuItem( "Medium" );
		level_hard 		= new JCheckBoxMenuItem( "Hard" );
		level_ultimate	= new JCheckBoxMenuItem( "Ultimate" );

		ButtonGroup	group = new ButtonGroup();
		group.add( level_easy );
		group.add( level_medium );
		group.add( level_hard );
		group.add( level_ultimate );
		
		level_menu.add( level_easy );
		level_menu.add( level_medium );
		level_menu.add( level_hard );
		level_menu.add( level_ultimate );
		
		menu_bar.add( level_menu );
		
			// symmetry
		
		JMenu	symmetry_menu = new JMenu( "Symmetry" );
		
		symmetry_menu.setMnemonic( 'S' );
		
		symmetry_none 		= new JCheckBoxMenuItem( "None", true );
		symmetry_horizontal = new JCheckBoxMenuItem( "Horizontal" );
		symmetry_vertical 	= new JCheckBoxMenuItem( "Vertical" );
		symmetry_square		= new JCheckBoxMenuItem( "Square" );

		ButtonGroup	symmetry_group = new ButtonGroup();
		symmetry_group.add( symmetry_none );
		symmetry_group.add( symmetry_horizontal );
		symmetry_group.add( symmetry_vertical );
		symmetry_group.add( symmetry_square );
		
		symmetry_menu.add( symmetry_none );
		symmetry_menu.add( symmetry_horizontal );
		symmetry_menu.add( symmetry_vertical );
		symmetry_menu.add( symmetry_square );
		
		menu_bar.add( symmetry_menu );
		
			// help
		
		JMenu	help_menu = new JMenu( "Help" );
		
		help_menu.setMnemonic( 'H' );

		JMenuItem	help_about = new JMenuItem( "About Sudoku" );

		help_menu.add( help_about );

		menu_bar.add( help_menu );
		
		help_about.addActionListener(
			new ActionListener()
			{
				public void
				actionPerformed(
					ActionEvent	ev )
				{
					about();
				}
			});
		
		main.add( 
				menu_bar,
				new SudokuUIGBC( 
						0, row_pos++, model.getWidth()+3, 1, 0, 0, 
						GridBagConstraints.EAST,
						GridBagConstraints.HORIZONTAL,
						new Insets( 0,0,8,0 ), 0, 0));
		
		JPanel	option_panel = new JPanel( new GridBagLayout());
		
		int	opt_pos = 0;
		
		final JCheckBox	show_poss = new JCheckBox( "Show possibilities", show_possibilities );
		
		show_poss.addActionListener(
				new ActionListener()
				{
					public void
					actionPerformed(
						ActionEvent	ev )
					{
						show_possibilities = show_poss.isSelected();
						
						main.repaint();
					}
				});
		
		option_panel.add( 
			show_poss,
			new SudokuUIGBC( 
				opt_pos++, 0, 1, 1, 0, 0, 
				GridBagConstraints.CENTER,
				GridBagConstraints.NONE,
				new Insets( 0,0,0,0 ), 0, 0));
		
		final JCheckBox	show_errors_cb = new JCheckBox( "Show errors", show_errors );
		
		show_errors_cb.addActionListener(
				new ActionListener()
				{
					public void
					actionPerformed(
						ActionEvent	ev )
					{
						show_errors = show_errors_cb.isSelected();
						
						main.repaint();
					}
				});
		
		option_panel.add( 
			show_errors_cb,
			new SudokuUIGBC( 
					opt_pos++, 0, 1, 1, 0, 0, 
					GridBagConstraints.CENTER,
					GridBagConstraints.NONE,
					new Insets( 0,0,0,0 ), 0, 0));
		
		final JCheckBox	auto_solve_cb = new JCheckBox( "Auto-solve", auto_solve );
		
		auto_solve_cb.addActionListener(
				new ActionListener()
				{
					public void
					actionPerformed(
						ActionEvent	ev )
					{
						try{
							auto_solve = auto_solve_cb.isSelected();
							
							main.repaint();
							
						}catch( Throwable e ){
							
							e.printStackTrace();
						}
					}
				});
		
		option_panel.add( 
				auto_solve_cb,
			new SudokuUIGBC( 
					opt_pos++, 0, 1, 1, 0, 0, 
					GridBagConstraints.CENTER,
					GridBagConstraints.NONE,
					new Insets( 0,0,0,0 ), 0, 0));
		
		JButton hint = new JButton( "Hint" );
		
		hint.addActionListener(
			new ActionListener()
			{
				public void
				actionPerformed(
					ActionEvent	ev )
				{
					model.getHint();
				}
			});
		
		option_panel.add( 
			hint,
			new SudokuUIGBC( 
					opt_pos++, 0, 1, 1, 0, 0, 
					GridBagConstraints.CENTER,
					GridBagConstraints.NONE,
					new Insets( 0,0,0,0 ), 0, 0));
		
		option_panel.add( 
				new JPanel(),
				new SudokuUIGBC( 
						opt_pos++, 0, 1, 1, 1.0, 0, 
						GridBagConstraints.CENTER,
						GridBagConstraints.HORIZONTAL,
						new Insets( 0,0,0,0 ), 0, 0));
		
		main.add( 
				option_panel,
				new SudokuUIGBC( 
						0, row_pos++, 2, 1, 0, 0, 
						GridBagConstraints.EAST,
						GridBagConstraints.HORIZONTAL,
						new Insets( 0,0,8,0 ), 0, 0));
		
		ui_values	= new SudokuUIValue[model.getWidth()][model.getHeight()];
		
		int	game_row_pos = 0;
		
		for (int x=0;x<model.getWidth();x++){
			
			int	game_col_pos = 0;

			if ( x != 0 && x % model.getSize() == 0 ){
			
				SudokuUIPanel	line = new SudokuUIPanel();
				
				line.setFixedSize(-1,2);
				
				line.setBorder( BorderFactory.createLineBorder( Color.black,1 ));
						
				game_panel.add( 
						line,
						new SudokuUIGBC( 
								game_col_pos, game_row_pos++, model.getWidth()+2, 1, 0, 0, 
								GridBagConstraints.CENTER,
								GridBagConstraints.HORIZONTAL,
								new Insets( 0,0,0,0 ), 0, 0));
			}
			
			for (int y=0;y<model.getHeight();y++){

				if ( y != 0 && y % model.getSize() == 0 ){
					
					SudokuUIPanel	line = new SudokuUIPanel();
					
					line.setFixedSize(2,-1);
					
					line.setBorder( BorderFactory.createLineBorder( Color.black,1 ));
							
					game_panel.add( 
							line,
							new SudokuUIGBC( 
									game_col_pos, game_row_pos, 1, 1, 0, 0, 
									GridBagConstraints.CENTER,
									GridBagConstraints.VERTICAL,
									new Insets( 0,0,0,0 ), 0, 0));
					
					game_col_pos++;
				}
				
				SudokuUIValue	pan = new SudokuUIValue( this, values[y][x]);
				
				ui_values[y][x]	= pan;
				
				pan.setFixedSize( 50, 50 );
				
				pan.setBorder( BorderFactory.createLineBorder(Color.LIGHT_GRAY ));
				
				game_panel.add( 
						pan,
						new SudokuUIGBC( 
								game_col_pos++, game_row_pos, 1, 1, 0, 0, 
								GridBagConstraints.CENTER,
								GridBagConstraints.NONE,
								new Insets( 0,0,0,0 ), 0, 0));
			}
						
			game_row_pos++;
		}
			

		main.add( 
				game_panel,
				new SudokuUIGBC( 
						1, row_pos, 1, 1, 1.0, 0, 
						GridBagConstraints.CENTER,
						GridBagConstraints.HORIZONTAL,
						new Insets( 0,0,0,0 ), 0, 0));

		row_pos++;
		
		status = new JLabel( "" );
		
		updateStatus();
		
		status.setBorder( BorderFactory.createLineBorder(Color.lightGray));
		
		main.add( 
				status,
				new SudokuUIGBC( 
						1, row_pos++, 1, 1, 1.0, 0.0, 
						GridBagConstraints.CENTER,
						GridBagConstraints.HORIZONTAL,
						new Insets( 4,0,0,0 ), 0, 0));
		main.add( 
				new JPanel(),
				new SudokuUIGBC( 
						0, row_pos, 2, 1, 1.0, 1.0, 
						GridBagConstraints.CENTER,
						GridBagConstraints.BOTH,
						new Insets( 0,0,0,0 ), 0, 0));
	}

	protected void
	generate()
	{
		final JDialog	dialog = new JDialog( JOptionPane.getFrameForComponent( main ), "Generation in progress", true );
	
		Container	container = dialog.getContentPane();
		
		container.setLayout( new BorderLayout());
		
		JPanel	panel = new JPanel( new GridBagLayout());
		
		container.add( panel, BorderLayout.CENTER );
		
		final JLabel	searched 	= new JLabel( "" );
		final JLabel	best	 	= new JLabel( "" );
		final JButton	stop		= new JButton( "Stop" );
		final JButton	restart		= new JButton( "Restart" );
		
		panel.add( 
			searched,
			new SudokuUIGBC( 
					0, 0, 3, 1, 0, 0, 
					GridBagConstraints.WEST,
					GridBagConstraints.NONE,
					new Insets( 0,0,0,0 ), 0, 0));
		
		panel.add( 
			best,
			new SudokuUIGBC( 
					0, 1, 3, 1, 0, 0, 
					GridBagConstraints.WEST,
					GridBagConstraints.NONE,
					new Insets( 0,0,0,0 ), 0, 0));
		
		panel.add( 
				new JPanel(),
				new SudokuUIGBC( 
						0, 2, 3, 1, 0.0, 1.0, 
						GridBagConstraints.CENTER,
						GridBagConstraints.VERTICAL,
						new Insets( 0,0,0,0 ), 0, 0));
		panel.add( 
			new JPanel(),
			new SudokuUIGBC( 
					0, 3, 1, 1, 1.0, 0, 
					GridBagConstraints.CENTER,
					GridBagConstraints.HORIZONTAL,
					new Insets( 0,0,0,0 ), 0, 0));
			
		panel.add( 
			restart, 
			new SudokuUIGBC( 
					1, 3, 1, 1, 0, 0, 
					GridBagConstraints.EAST,
					GridBagConstraints.NONE,
					new Insets( 0,0,0,0 ), 0, 0));

		panel.add( 
				stop, 
				new SudokuUIGBC( 
						2, 3, 1, 1, 0, 0, 
						GridBagConstraints.EAST,
						GridBagConstraints.NONE,
						new Insets( 0,0,0,0 ), 0, 0));
		
		final Semaphore	stop_sem	= new Semaphore();
		final Semaphore	stopped_sem	= new Semaphore();
		
		stop.addActionListener(
			new ActionListener()
			{
				public void
				actionPerformed(
					ActionEvent	ev )
				{
					stop_sem.release();
				}
			});
		
		restart.addActionListener(
				new ActionListener()
				{
					public void
					actionPerformed(
						ActionEvent	ev )
					{
						stop_sem.release();
					
						new Thread()
						{
							public void
							run()
							{
								stopped_sem.reserve();
						
								generate();
							}
						}.start();
					}
				});
		
		dialog.addWindowListener(
			new WindowAdapter()
			{
				  public void 
				  windowClosing(
					WindowEvent e)
				  {
					  stop_sem.release();
				  }

			});
		
		dialog.setPreferredSize( new Dimension( 300, 120 ));
		
		dialog.pack();
		
		Dimension 	main_size 	= main.getSize();
		
		Dimension	dialog_size = dialog.getSize();
	
		dialog.setLocation(
				main.getLocation().x + ( main_size.width - dialog_size.width  )/2,
				main.getLocation().y + ( main_size.height - dialog_size.height  )/2);
	
		dialog.setLocationRelativeTo( main );
				
		new Thread()
		{
			public void
			run()
			{
				dialog.setVisible( true );
			}
		}.start();
		
		new Thread()
		{
			public void
			run()
			{
				try{
					setUIUpdateEnabled( false );
					
					int	symmetry;
					
					if ( symmetry_horizontal.isSelected()){
						
						symmetry = SudokuModel.SYM_HORIZONTAL;
						
					}else if ( symmetry_vertical.isSelected()){
						
						symmetry = SudokuModel.SYM_VERTICAL;
						
					}else if ( symmetry_square.isSelected()){
						
						symmetry = SudokuModel.SYM_SQUARE;
						
					}else{
						
						symmetry = SudokuModel.SYM_NONE;
					}
					
					model.generate(
						level_ultimate.isSelected(),
						symmetry,
						new SudokuModelGenerationListener()
						{
							int		num 	= 0;
							int		max	= 0;
							
							public boolean
							generated()
							{
								num++;
								
								searched.setText( "Searched: " + num );
								
								SudokuModelComplexity	complexity = model.getComplexity();
								
								int	comp = complexity.getComplexityMetric();
								
								if ( comp > max ){
									
									max	= comp;
								
									best.setText( "Level: " + complexity.getString( false ));
									
									if ( level_easy.isSelected()){
										
										if ( complexity.getValues() <= 28 ){
											
											return( false );
										}
									}else if ( level_medium.isSelected()){
										
										if (	complexity.getValues() <= 28 && 
												complexity.getComplexRuleCount() > 0 ){
											
											return( false );
										}
									}
								}
								
								return( stop_sem.getValue() == 0 );
							}
						});
					
				}catch( Throwable e ){
					
					e.printStackTrace();
					
				}finally{
					
					setUIUpdateEnabled( true );
					
					dialog.dispose();
					
					stopped_sem.release();
					
					SwingUtilities.invokeLater(
						new Runnable()
						{
							public void
							run()
							{
								main.repaint();
							}
						});
				}
			}
		}.start();
	}
	
	protected void
	about()
	{
		final JDialog	dialog = new JDialog( JOptionPane.getFrameForComponent( main ), "About Sudoku", false );
	
		Container	container = dialog.getContentPane();
		
		container.setLayout( new BorderLayout());
		
		JPanel	panel = new JPanel( new GridBagLayout());
		
		container.add( panel, BorderLayout.CENTER );
			
		JLabel info = 
			new JLabel( 
				"<HTML><CENTER>Sudoku game: written by Paul Gardner</CENTER><BR>" +
					"<UL>" +	
					"<LI>Create a new puzzle via the File->Generate menu" +
						"<UL>Set the Level and Symmetry before generating</UL>" +
					"</LI>" +
					"<LI>Navigate the squares with mouse or arrow keys. Enter values via:" +
						"<UL>" +
							"<LI>Number - enter value; delete key to remove</LI>"+
							"<LI>Shift+number - enter starting value; Shift+delete to remove</LI>" +
							"<LI>Ctrl/Alt+number - annotate guess: green = possible, red = not possible; repeat to cycle</LI>" +
						"</UL>" +
					"</LI>" +
					"<LI>Note that ultimate/hard-level generation requires an explicit stop</LI>" +
					"</UL>"+
				"</HTML>");
		
		JTextArea	detail = new JTextArea( model.getComplexity().getString( true ));
		
		detail.setEditable( false );
		
		panel.add( 
			info,
			new SudokuUIGBC( 
					0, 0, 1, 1, 1.0, 1.0, 
					GridBagConstraints.CENTER,
					GridBagConstraints.BOTH,
					new Insets( 0,0,0,0 ), 0, 0));
		
		panel.add( 
			new JScrollPane( detail ),
			new SudokuUIGBC( 
					0, 1, 1, 1, 1.0, 1.0, 
					GridBagConstraints.CENTER,
					GridBagConstraints.BOTH,
					new Insets( 0,0,0,0 ), 0, 0));
		
		
				
		dialog.setPreferredSize( new Dimension( 500, 400 ));
		
		dialog.pack();
		
		Dimension 	main_size 	= main.getSize();
		
		Dimension	dialog_size = dialog.getSize();
	
		dialog.setLocation(
				main.getLocation().x + ( main_size.width - dialog_size.width  )/2,
				main.getLocation().y + ( main_size.height - dialog_size.height  )/2);
	
		dialog.setLocationRelativeTo( main );
				

			dialog.setVisible( true );
	}
	
	protected void
	updateStatus()
	{
		status.setText( "Level: " + model.getComplexity().getString( false ));
	}
	
	protected void
	setUIUpdateEnabled( 
		boolean	enabled )
	{
		for (int x=0;x<ui_values.length;x++){
			
			SudokuUIValue[]col = ui_values[x];
			
			for (int j=0;j<col.length;j++){
				
				col[j].enableUpdate( enabled );
			}
		}
	}
	
	public void
	requestFocus()
	{
		ui_values[0][0].requestFocus();
	}
	
	public JPanel
	getComponent()
	{
		return( main );
	}
	
	protected boolean
	showPossibilities()
	{
		return( show_possibilities );
	}
	
	protected boolean
	showErrors()
	{
		return( show_errors );
	}
	
	protected boolean
	isAutoSolve()
	{
		return( auto_solve );
	}
	
	protected void
	moveRequested(
		SudokuValue		value,
		int				x,
		int				y )
	{
		int	new_x = value.getX() + x;
		int new_y = value.getY() + y;
		
		if ( 	new_x >= 0 && new_x < model.getWidth() &&
				new_y >= 0 && new_y < model.getHeight()){
			
			ui_values[new_x][new_y].requestFocus();
		}
	}
	
	public void
	modelChanged()
	{
		updateStatus();
		
		main.repaint();
	}
	
	public void
	destroy()
	{
		for (int x=0;x<ui_values.length;x++){
			
			SudokuUIValue[]col = ui_values[x];
			
			for (int j=0;j<col.length;j++){
				
				col[j].destroy();
			}
		}
	}
}
