/*
 * Created on 04-Sep-2005
 * Created by Paul Gardner
 * Copyright (C) 2005 Aelitis, All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * AELITIS, SARL au capital de 30,000 euros
 * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
 *
 */

package org.parg.azureus.plugins.sudoku.ui.swing;

import java.awt.*;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.font.GlyphVector;
import java.awt.geom.Rectangle2D;

import org.parg.azureus.plugins.sudoku.model.SudokuPossibilityMetric;
import org.parg.azureus.plugins.sudoku.model.SudokuSolveInstance;
import org.parg.azureus.plugins.sudoku.model.SudokuValueListener;
import org.parg.azureus.plugins.sudoku.model.SudokuValue;
import org.parg.azureus.plugins.sudoku.model.SudokuValueMetric;

public class 
SudokuUIValue
	extends SudokuUIPanel
	implements SudokuValueListener
{	
	private SudokuUIMain	ui;
	private SudokuValue		value;
	private boolean			enabled	= true;
	
	private boolean			was_hint;
	private Color			background	= Color.WHITE;
		
	public
	SudokuUIValue(
		SudokuUIMain	_ui,
		SudokuValue		_value )
	{
		ui			= _ui;
		value		= _value;
		
		value.addListener( this );
		
		init();
	}
	
	public boolean
	isFocusTraversable()
	{
		return( true );
	}
	
	protected void
	init()
	{
		addKeyListener( 
			new KeyAdapter()
			{
				public void 
				keyTyped(
					KeyEvent e)
				{
					}
				
				public void 
				keyPressed(
					KeyEvent e)
				{
					int[]	map1 = {
							KeyEvent.VK_1,KeyEvent.VK_2,KeyEvent.VK_3,
							KeyEvent.VK_4,KeyEvent.VK_5,KeyEvent.VK_6,
							KeyEvent.VK_7,KeyEvent.VK_8,KeyEvent.VK_9 };
					
					int[]	map2 = { 
						KeyEvent.VK_NUMPAD1,KeyEvent.VK_NUMPAD2,KeyEvent.VK_NUMPAD3,
						KeyEvent.VK_NUMPAD4,KeyEvent.VK_NUMPAD5,KeyEvent.VK_NUMPAD6,
						KeyEvent.VK_NUMPAD7,KeyEvent.VK_NUMPAD8,KeyEvent.VK_NUMPAD9 };

					int[][]	maps = {map1,map2};
					
					int	code = e.getKeyCode();
					
					for (int m=0;m<maps.length;m++){
						int[]	map = maps[m];
					
						for (int i=0;i<map.length;i++){
							
							if ( map[i] == code ){
								
								if ( value.isCore()){
									
									if ( !e.isShiftDown()){
										
										return;
									}
								}
								
								if ( e.isShiftDown()){
										
									value.setCore( true );
								}
								
								if ( e.isControlDown() || e.isAltDown()){
								
									int	ot = value.getPossibleOverrides()[i];
									
									if ( ot == SudokuValue.OT_NONE ){
										
										ot = SudokuValue.OT_TRUE;
										
									}else if ( ot == SudokuValue.OT_TRUE ){
										
										ot = SudokuValue.OT_FALSE;
									}else{
										
										ot = SudokuValue.OT_NONE;
									}
									
									value.setPossibleOverride( i+1, ot);
								}else{
								
									value.setValue( i+1, true );
								}
								
								return;
							}
						}
					}
					
					if ( 	code == KeyEvent.VK_DELETE ||
							code == KeyEvent.VK_BACK_SPACE ){
						
						if ( value.isCore()){
							
							if ( !e.isShiftDown()){
								
								return;
							}
						}
						
						if ( value.isHint()){
							
							value.clearHint();
							
						}else{
							
							value.setCore( false );
						
							value.setValue( -1, true );
						}					
				
					}else if ( code == KeyEvent.VK_UP ){
										
						ui.moveRequested( value, 0, -1 );
					
					}else if ( code == KeyEvent.VK_DOWN ){
						
						ui.moveRequested( value, 0, 1 );
						
					}else if ( code == KeyEvent.VK_LEFT ){
						
						ui.moveRequested( value, -1, 0 );
						
					}else if ( code == KeyEvent.VK_RIGHT ){
						
						ui.moveRequested( value, 1, 0 );
					}
				}
			});
		
		addMouseListener(
			new MouseAdapter()
			{
				public void 
				mouseClicked(
					MouseEvent e)
				{
					requestFocus();
				}

			});
		
		addFocusListener(
			new FocusListener()
			{
				public void 
				focusGained(
					FocusEvent e)
				{
			    	background = Color.YELLOW;
			    	
			    	repaint();
				}

			    public void 
			    focusLost(
			    	FocusEvent e)
			    {
			    	background = Color.WHITE;
			    	
			    	repaint();
			    }
			});
	}
	
	protected void
	destroy()
	{
		value.removeListener( this );
	}

	public void
	valueChanged(
		SudokuSolveInstance		solve_instance,
		SudokuValue				value )
	{
		if ( enabled ){
			
			repaint();
		}
	}
	
	public void
	possibilityChanged(
		SudokuSolveInstance	solve_instance,
		SudokuValue			value,
		int					val,
		boolean				possible )
	{
		if ( enabled ){
			
			repaint();
		}
	}
	
	public void
	otherChanged(
		SudokuValue			value )
	{
		if ( enabled ){
			
			repaint();
		}	
	}
	
	protected void 
	enableUpdate(
		boolean	e )
	{
		enabled	= e;
	}
	
	public void
	paintComponent(
		Graphics	_g )
	{
     	Dimension	size = getSize();
      	
		Graphics2D	g = (Graphics2D)_g;
		
		super.paintComponent(_g);
		
		g.setColor( background );
		
		g.fillRect( 0, 0, size.width, size.height );
		
		if ( !enabled ){
			
			return;
		}
		
		g.setColor( Color.black );
		
    	g.setRenderingHint(
    			RenderingHints.KEY_ANTIALIASING,
    			RenderingHints.VALUE_ANTIALIAS_ON);
      	
      	g.setRenderingHint(
      			RenderingHints.KEY_TEXT_ANTIALIASING,
    			RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

      	g.setRenderingHint( 
      			RenderingHints.KEY_FRACTIONALMETRICS, 
      			RenderingHints.VALUE_FRACTIONALMETRICS_ON );

       	String	str_val = "";
       	
		boolean	auto_solve = ui.isAutoSolve();
		
		boolean	hint_change = value.isHint() != was_hint;
			
		int	value_shown = -1;
		
		if ( hint_change ){
			
			was_hint = value.isHint();
			
			if ( was_hint ){
				
				requestFocus();
			}
		}
		
		if ( auto_solve || value.isExplicit() || value.isHint()){
			
			if ( value.hasValue()){
				
		     	int	val = value.getValue();
	
		     	value_shown	= val;
		     	
				str_val =  "" + val;
				
				if ( !value.isValid() && 
						( auto_solve || ui.showErrors())){
					
					g.setColor( Color.RED );
					
				}else if ( value.isHint()){
					
					g.setColor( Color.GREEN.darker() );
					
				}else if ( !value.isExplicit()){
					
					g.setColor( Color.BLUE );
					
				}else if ( value.isCore()){
					
					g.setColor( Color.BLACK );
	
				}else{
					
					g.setColor( Color.GRAY );
				}
			}else{
				
				if ( !value.isValid()){
					
					str_val = "?";
					
					g.setColor( Color.RED );
				}
			}
			
		     	 
			g.setFont( new Font( "Helvetica", Font.BOLD, 24));
			 	
			GlyphVector text_gv = g.getFont().createGlyphVector(g.getFontRenderContext(), str_val );
					
			Rectangle2D text_rect = text_gv.getVisualBounds();	
				
			g.drawGlyphVector( 
					text_gv, 
					(float)(size.width - text_rect.getWidth())/2,
					(float)(size.height - ( size.height - text_rect.getHeight())/2));
		}
			
		boolean[]	possibles = null;
		
		int[]	possible_overrides = value.getPossibleOverrides();

		if ( value.isHint()){
			
			possibles = value.getHintPossibles();
			
		}else if ( auto_solve ){
			
			possibles = value.getPossibles( false );
	
		}else if (  ui.showPossibilities()){
			
			if ( !value.isCore()){
				
				possibles = value.getHintPossibles();
			}
		}else{
			
			for (int i=0;i<possible_overrides.length;i++){
				
				if ( possible_overrides[i] != SudokuValue.OT_NONE ){
					
					possibles = new boolean[possible_overrides.length];
					
					break;
				}
			}
		}
		
		if ( possibles != null ){
				
			SudokuPossibilityMetric[]	reasons;
			
			if ( value.isHint()){
			
				reasons =  value.getHintPossibleReasons();
				
			}else{
				
				reasons =  value.getPossibleReasons();
			}
			
			String	tip = "<HTML><BODY>";
			
			SudokuValueMetric	value_metric = value.getValueMetric();
			
			if ( auto_solve || value.isHint()){
				
				tip += "Value: " + (value_metric==null?"Unknown":value_metric.getReason()) + "<BR>";
			}
			
			for (int i=0;i<reasons.length;i++){
				
				int	ot =  possible_overrides[i];
				
				if ((!possibles[i]) || ot != SudokuValue.OT_NONE ){
					
					if ( reasons[i] != null ){
					
						tip += (i+1) + ") " + reasons[i].getReason() + "<BR>";
						
					}else if ( ot ==  SudokuValue.OT_FALSE ){
						
						tip += (i+1) + ") FALSE Override<BR>";
						
					}else if ( ot ==  SudokuValue.OT_TRUE ){
						
						tip += (i+1) + ") TRUE Override<BR>";
					}
				}
			}
			
			tip += "</BODY></HTML>";
			
			setToolTipText(tip );
						
		    g.setFont( new Font( "Helvetica", Font.PLAIN, 8));
	
		    GlyphVector test_gv = g.getFont().createGlyphVector(g.getFontRenderContext(), "8" );
			
			Rectangle2D possible_rect = test_gv.getVisualBounds();
			
			boolean	show_p = ui.showPossibilities();
			
			for (int i=0;i<possibles.length;i++){
				
					// don't show overrides if we're showing all of them
				
				int	ot =  show_p?SudokuValue.OT_NONE:possible_overrides[i];
				
				if ( possibles[i] || ot != SudokuValue.OT_NONE ){
					
					if ( value_shown == (i+1) && ot == SudokuValue.OT_NONE ){
						
						continue;
					}
					
					if ( ot == SudokuValue.OT_TRUE ){
					
						g.setColor( Color.GREEN.darker());
						
					}else if ( ot == SudokuValue.OT_FALSE ){
						
						g.setColor( Color.RED );
						
					}else{
						
						g.setColor( Color.BLACK );
					}
					
				    GlyphVector possible_gv = g.getFont().createGlyphVector(g.getFontRenderContext(), "" + (i+1) );
	
					g.drawGlyphVector( 
							possible_gv, 
							(float)( 4+i*(possible_rect.getWidth()+1)),
							(float)( 4+possible_rect.getHeight()));
	
				}
			}
		}else{
			
			setToolTipText( null );
		}
	}
}
