/*
 * Created on 05-Sep-2005
 * Created by Paul Gardner
 * Copyright (C) 2005 Aelitis, All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * AELITIS, SARL au capital de 30,000 euros
 * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
 *
 */

package org.parg.azureus.plugins.sudoku.ui.swt;


import java.awt.BorderLayout;
import java.awt.Container;

import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;

import org.gudy.azureus2.ui.swt.plugins.UISWTInstance;
import org.gudy.azureus2.ui.swt.plugins.UISWTView;
import org.gudy.azureus2.ui.swt.plugins.UISWTViewEvent;
import org.gudy.azureus2.ui.swt.plugins.UISWTViewEventListener;
import org.parg.azureus.plugins.sudoku.model.SudokuModel;
import org.parg.azureus.plugins.sudoku.ui.swing.SudokuUIAdapter;
import org.parg.azureus.plugins.sudoku.ui.swing.SudokuUIMain;

public class 
SudokuUISWT 
{
	private UISWTInstance			swt;
		
	public
	SudokuUISWT(
		final SudokuModel		model,
		UISWTInstance			_swt )
	{
		swt		= _swt;
		
		swt.addView(
			UISWTInstance.VIEW_MAIN,
			"aesudoku",
			new UISWTViewEventListener()
			{
				private SudokuUIMain	sudoku_ui;

				public boolean 
				eventOccurred(
					UISWTViewEvent event )
				{
					switch( event.getType()){
					
						case UISWTViewEvent.TYPE_CREATE:{
						
							event.getView().setControlType( UISWTView.CONTROLTYPE_AWT );
						
							break;
						}
						case UISWTViewEvent.TYPE_INITIALIZE:{
						
							final	UISWTView view = event.getView();
							
							sudoku_ui = 
								new SudokuUIMain( 
										model,
										new SudokuUIAdapter()
										{
											public void
											closed()
											{
												view.closeView();
											}
										});
							
							Container container = (Container)event.getData();
							
							container.setLayout( new BorderLayout());
							
							container.add( new JScrollPane( sudoku_ui.getComponent()), BorderLayout.CENTER );
							
							break;
						}
						case UISWTViewEvent.TYPE_FOCUSGAINED:{
						
							SwingUtilities.invokeLater(
								new Runnable()
								{
									public void
									run()
									{
										sudoku_ui.requestFocus();
									}
								});						
							break;
						}
						case UISWTViewEvent.TYPE_DESTROY:{
						
							if (sudoku_ui != null) {
								sudoku_ui.destroy();
							}
													
							break;
						}
					}
					
					return( true );
				}
			});
	}

	public void
	destroy()
	{
		swt.removeViews( UISWTInstance.VIEW_MAIN, "aesudoku" );
	}
}
